@@ -1,17 +1,30 @@
 package App::gimpgitbuild::Command::build;
-$App::gimpgitbuild::Command::build::VERSION = '0.20.0';
+$App::gimpgitbuild::Command::build::VERSION = '0.22.0';
 use strict;
 use warnings;
+use autodie;
 use 5.014;
 
 use App::gimpgitbuild -command;
 
 use File::Which qw/ which /;
-use Path::Tiny qw/ path tempdir tempfile cwd /;
+use Path::Tiny qw/ path cwd /;
 
 use App::gimpgitbuild::API::GitBuild ();
 use Git::Sync::App                   ();
 
+sub _mode
+{
+    my $self = shift;
+
+    if (@_)
+    {
+        $self->{_mode} = shift;
+    }
+
+    return $self->{_mode};
+}
+
 sub _api_obj
 {
     my $self = shift;
@@ -24,6 +37,18 @@ sub _api_obj
     return $self->{_api_obj};
 }
 
+sub _process_executor
+{
+    my $self = shift;
+
+    if (@_)
+    {
+        $self->{_process_executor} = shift;
+    }
+
+    return $self->{_process_executor};
+}
+
 sub description
 {
     return "build gimp from git";
@@ -36,7 +61,8 @@ sub abstract
 
 sub opt_spec
 {
-    return ( [ "mode=s", "Mode (e.g: \"clean\")" ], );
+    return ( [ "mode=s", "Mode (e.g: \"clean\")" ],
+        [ "process-exe=s", qq#Process executor (= "sh" or "perl")#, ] );
 
 
 }
@@ -90,8 +116,10 @@ sub _git_build
 {
     my $self                 = shift;
     my $args                 = shift;
+    my $orig_cwd             = cwd()->absolute();
     my $id                   = $args->{id};
     my $extra_configure_args = ( $args->{extra_configure_args} // [] );
+    my $SHELL_PREFIX         = "set -e -x";
 
     if ( defined($skip_builds_re) and $id =~ $skip_builds_re )
     {
@@ -112,24 +140,91 @@ sub _git_build
         _do_system( { cmd => [qq#git clone "$args->{url}" "$git_co"#] } );
     }
 
-    my $meson_build_shell_cmd =
-qq#mkdir -p "$BUILD_DIR" && cd "$BUILD_DIR" && meson --prefix="$args->{prefix}" $UBUNTU_MESON_LIBDIR_OVERRIDE .. && ninja $PAR_JOBS && ninja $PAR_JOBS test && ninja $PAR_JOBS install#;
-    my $autoconf_build_shell_cmd =
-qq#NOCONFIGURE=1 ./autogen.sh && mkdir -p "$BUILD_DIR" && cd "$BUILD_DIR" && ../configure @{$extra_configure_args} --prefix="$args->{prefix}" && make $PAR_JOBS && @{[_check()]} && make install#;
+    my $_autodie_chdir = sub {
+        my $dirname = shift;
+        if ( not chdir($dirname) )
+        {
+            die qq#Failed changing directory to "$dirname"!#;
+        }
+        return;
+    };
+    my $shell_cmd = sub {
+        return shift;
+    };
+    my $chdir_cmd = sub {
+        return $shell_cmd->( qq#cd "# . shift(@_) . qq#"# );
+    };
+    my $PERL_EXECUTE = ( $self->_process_executor() eq 'perl' );
+    if ($PERL_EXECUTE)
+    {
+        $shell_cmd = sub {
+            my $cmd = shift;
+            return sub {
+                return _do_system(
+                    {
+                        cmd => ["$SHELL_PREFIX ; $cmd"],
+                    }
+                );
+            };
+        };
+        $chdir_cmd = sub {
+            my $dirname = shift;
+            return sub {
+                return $_autodie_chdir->($dirname);
+            };
+        };
+    }
+
+    my @meson_build_shell_cmd = (
+        $shell_cmd->(qq#mkdir -p "$BUILD_DIR"#),
+        $chdir_cmd->($BUILD_DIR),
+        $shell_cmd->(
+qq#meson --prefix="$args->{prefix}" $UBUNTU_MESON_LIBDIR_OVERRIDE ..#
+        ),
+        $shell_cmd->(qq#ninja $PAR_JOBS#),
+        $shell_cmd->(qq#ninja $PAR_JOBS test#),
+        $shell_cmd->(qq#ninja $PAR_JOBS install#)
+    );
+    my @autoconf_build_shell_cmd = (
+        $shell_cmd->(qq#NOCONFIGURE=1 ./autogen.sh#),
+        $shell_cmd->(qq#mkdir -p "$BUILD_DIR"#),
+        $chdir_cmd->($BUILD_DIR),
+        $shell_cmd->(
+            qq#../configure @{$extra_configure_args} --prefix="$args->{prefix}"#
+        ),
+        $shell_cmd->(qq#make $PAR_JOBS#),
+        $shell_cmd->(qq#@{[_check()]}#),
+        $shell_cmd->(qq#make install#)
+    );
+    my @clean_mode_shell_cmd = ( $shell_cmd->(qq#git clean -dxf .#) );
     my $sync_cmd = $self->_git_sync( { branch => $args->{branch}, } );
-    my $run      = sub {
+    my @commands = (
+        $chdir_cmd->($git_co),
+        $shell_cmd->(qq#git checkout "$args->{branch}"#),
+        $shell_cmd->(qq#( $args->{tag} || $sync_cmd )#),
+        (
+            ( $self->_mode() eq 'clean' ) ? @clean_mode_shell_cmd
+            : (
+                  $args->{use_meson} ? @meson_build_shell_cmd
+                : @autoconf_build_shell_cmd
+            )
+        ),
+    );
+
+    my $run = sub {
+        if ($PERL_EXECUTE)
+        {
+            foreach my $cb (@commands)
+            {
+                $cb->();
+            }
+            return;
+        }
+        my $aggregate_shell_command =
+            "$SHELL_PREFIX ; " . join( " ; ", @commands );
         return _do_system(
             {
-                cmd => [
-qq#cd "$git_co" && git checkout "$args->{branch}" && ( $args->{tag} || $sync_cmd ) && #
-                        . (
-                        ( $self->{mode} eq 'clean' ) ? "git clean -dxf ."
-                        : (
-                              $args->{use_meson} ? $meson_build_shell_cmd
-                            : $autoconf_build_shell_cmd
-                        )
-                        )
-                ]
+                cmd => [ $aggregate_shell_command, ]
             }
         );
     };
@@ -150,6 +245,7 @@ qq#cd "$git_co" && git checkout "$args->{branch}" && ( $args->{tag} || $sync_cmd
             $on_failure->( { exception => $Err, }, );
         }
     }
+    $_autodie_chdir->($orig_cwd);
     return;
 }
 
@@ -189,6 +285,16 @@ sub execute
         die "Unsupported mode '$mode'!";
     }
 
+    my $_process_executor = ( $opt->{process_exe} || 'perl' );
+    if (
+        not(   ( $_process_executor eq 'sh' )
+            or ( $_process_executor eq 'perl' ) )
+        )
+    {
+        die "Unsupported process-exe '$_process_executor'!";
+    }
+    $self->_process_executor($_process_executor);
+
     my $fh  = \*STDIN;
     my $obj = App::gimpgitbuild::API::GitBuild->new;
     $self->_api_obj($obj);
@@ -200,7 +306,7 @@ sub execute
     $ENV{XDG_DATA_DIRS}   = $env->{XDG_DATA_DIRS};
     _which_xvfb_run();
     _ascertain_lack_of_gtk_warnings();
-    $self->{mode} = $mode;
+    $self->_mode($mode);
 
     my $GNOME_GIT = 'https://gitlab.gnome.org/GNOME';
     $self->_git_build(
@@ -290,7 +396,7 @@ __END__
 
 =head1 VERSION
 
-version 0.20.0
+version 0.22.0
 
 =begin foo return (
         [ "output|o=s", "Output path" ],
